//
// dbresult.cpp
//

#include <string>
#include <iostream>
using namespace std;

#include <mysql/mysql.h>

#include "database.h"
#include "dbresult.h"

DBResult::DBResult (  )
{
	result = NULL;
	haveError = false;
}

DBResult::DBResult ( MYSQL * mysql, char *query )
{
	result = NULL;
	haveError = false;
	init ( mysql, query );
}

DBResult::~DBResult (  )
{
	if ( result != NULL )					// Free memory resources
		mysql_free_result ( result );
}

void DBResult::init ( MYSQL * mysql, char *query )
{
	if ( result != NULL )
	{
		mysql_free_result ( result );
		result = NULL;
	}
	if ( mysql_query ( mysql, query ) == 0 )
	{															// query OK
		result = mysql_store_result ( mysql );
		if ( result == NULL )
		{														// empty query
			if ( mysql_num_fields ( mysql ) == 0 )
				haveError = false;
			else
				haveError = true;
		}
		else
			haveError = false;
	}
	else
		haveError = true;
}

int DBResult::status (  )
{
	if ( haveError )
		return DB_ERROR;
	if ( result == NULL )
		return DB_COMMAND_OK;
	return DB_TUPLES_OK;
}

int DBResult::nrTuples (  )
{
	if ( result == NULL )
		return 0;
	return mysql_num_rows ( result );
}

int DBResult::nrFields (  )
{
	if ( result == NULL )
		return 0;
	return mysql_num_fields ( result );
}

char *DBResult::fieldName ( int n )
{
	MYSQL_FIELD *field;

	if ( result == NULL )
		return NULL;
	mysql_field_seek ( result, n );
	field = mysql_fetch_field ( result );
	if ( field == NULL )
		return NULL;
	return field->name;
}

int DBResult::fieldSize ( int n )
{
	MYSQL_FIELD *field;

	if ( result == NULL )
		return 0;
	mysql_field_seek ( result, n );
	field = mysql_fetch_field ( result );
	if ( field == NULL )
		return 0;
	return field->length;
}

int DBResult::fieldSize ( char *name )
{
	int i;

	if ( result == NULL )
		return 0;
	for ( i = 0; i < nrFields (  ); i++ )
		if ( !strcmp ( name, fieldName ( i ) ) )
			return fieldSize ( i );
	return 0;
}

void DBResult::seekTuple ( int tuple )
{
	if ( result == NULL )
		return;
	mysql_data_seek ( result, tuple );
}

char **DBResult::getTuple (  )
{
	MYSQL_ROW row;

	if ( result == NULL )
		return NULL;
	row = mysql_fetch_row ( result );
	return row;
}

char **DBResult::getTuple ( int tuple )
{
	seekTuple ( tuple );
	return getTuple (  );
}
